from rocolib.api.components.Component import Component
from rocolib.api.Function import Function

# ESP32 STACK HOLDER specify in dimensions.py if different number of stacks
# This is for 3 stacks
c = Component()

c.addParameter("length", 60, paramType="length")
c.addParameter("brains", "esp32stack", paramType="dimension") #change in dimensions.py
c.addParameter("heightChanger", 0, paramType="length")

#Cover/body of the car (I needed one that folds -90; will try on SimpleRectBeam)
for i in range(3):
    if i % 2 == 0:
        c.addSubcomponent("rect%d" %i, "Rectangle")
        c.addConstraint(("rect%d" %i, "l"), ("brains", "heightChanger"), "getDim(x[0], 'height') + x[1]")
        c.addConstraint(("rect%d" %i, "w"), "length")

for i in range(3, 5):
    c.addSubcomponent("rect%d" %i, "Rectangle")
    c.addConstraint(("rect%d" %i, "l"), "brains", "getDim(x, 'width')")
    c.addConstraint(("rect%d" %i, "w"), "length")

#TODO: ADD TABS FOR ATTACHMENT
# c.addConnection(("rect2", "b"), ("rect3", "t"), angle=90)
c.addConnection(("rect3", "r"), ("rect0", "r"), angle=-90)
c.addConnection(("rect2", "r"), ("rect4", "r"), angle=-90)
c.addConnection(("rect4", "l"), ("rect0", "l"), angle=-90)

# Putting the pins of the ESP32 Stack
c.addSubcomponent("header", "Header")
c.addConstraint(("header", "nrows"), "brains", "getDim(x, 'nrows')")
c.addConstraint(("header", "ncols"), "brains", "getDim(x, 'ncols')")
c.addConstraint(("header", "rowsep"), "brains", "getDim(x, 'rowsep')")
c.addConstraint(("header", "colsep"), "brains", "getDim(x, 'colsep')")

c.addConnection(("rect4", "face"),
                ("header", "decoration"),
                mode="hole", offset=Function(params=("length", "brains"), fnstring="(0, 0)"))

c.addConnection(("rect3", "face"),
                ("header", "decoration"),
                mode="hole", offset=Function(params=("length", "brains"), fnstring="(0, 0)"))

# Holes for servo and power supply
c.addParameter("dy", 18, parameterType="length")

for i in range(2):
    c.addSubcomponent("accessoryHole%d" %i, "Cutout")
    c.addConstraint(("accessoryHole%d" %i, "dy"), "brains", "getDim(x, 'height')/1.75")
    c.addConstraint(("accessoryHole%d" %i, "dx"), "dy", "x-3")

c.addConnection(("rect2", "face"),
                ("accessoryHole0", "decoration"),
                mode="hole", rotate=True,
                offset=Function(params=("brains", "length"), fnstring="(getDim(x[0], 'height') * 0.15, x[1] * 0.25)"))

c.addConnection(("rect0", "face"),
                ("accessoryHole1", "decoration"),
                mode="hole", rotate=True,
                offset=Function(params=("brains", "length"), fnstring="(getDim(x[0], 'height') * 0.15, -x[1] * 0.25)"))

for i in range(5):
    if i == 1:
        continue
    else:
        c.inheritAllInterfaces("rect%d" %i)

c.toLibrary("ESP32Stack")