import rocolib.utils.numsym as np
from rocolib.utils.transforms import RotateZ, Translate


def prefix(s1, s2):
    if s1 and s2:
        return s1 + "." + s2
    return s1 or s2


def tryImport(module, attribute):
    try:
        mod = __import__(module, fromlist=[attribute])
        obj = getattr(mod, attribute)
        return obj
    except ImportError:
        mod = __import__("rocolib.library." + module, fromlist=[attribute])
        obj = getattr(mod, attribute)
        return obj


def decorateGraph(face, decoration, offset=(0, 0), rotate=False, mode=None):
    try:
        dfaces = decoration.faces
    except AttributeError:
        dfaces = [decoration]

    if mode is None:
        mode = "hole"

    if rotate is False:
        rotate = 0
    elif rotate is True:
        rotate = -90

    for f in dfaces:
        t2d = transformDecorations(face, f.pts2d, offset=offset, rotate=rotate, mode=mode)
        f.transform2D = t2d
        f.addFace(face, np.inv(t2d))


def transformDecorations(face, pts2d, offset=(0, 0), rotate=0, flip=False, mode=None):
    a = np.deg2rad(rotate)
    c = np.cos(a)
    s = np.sin(a)

    face.addDecoration(([
                            (c * p[0] - s * p[1] + offset[0], s * p[0] + c * p[1] + offset[1])
                            for p in pts2d], mode))

    return np.dot(Translate([offset[0], offset[1], 0]), RotateZ(rotate))


def copyDecorations(self, deco_1, deco_2):
    (ni1, (sc1, i1, p1a, p1b)) = deco_1
    (ni2, (sc2, i2, p2a, p2b)) = deco_2

    self.inheritInterface(ni1, (sc1, i1))
    self.inheritInterface(ni2, (sc2, i2))

    f1 = self.getInterface(ni1).getFace()
    f2 = self.getInterface(ni2).getFace()

    p1o = f1.pts2d[p1a]
    p1x = f1.pts2d[p1b]
    p2o = f2.pts2d[p2a]
    p2x = f2.pts2d[p2b]

    a1 = np.arctan2(p1x[1] - p1o[1], p1x[0] - p1o[0])
    a2 = np.arctan2(p2x[1] - p2o[1], p2x[0] - p2o[0])

    for pts, mode in f1.decorations:
        transformDecorations(
            f2,
            [(px - p1o[0], py - p1o[1]) for (px, py) in pts],
            offset=p2o,
            rotate=np.rad2deg(a2 - a1),
            mode=mode
        )


def copyDecorations2(self, deco_1, deco_2, angle):
    (ni1, (sc1, i1, p1a)) = deco_1
    (ni2, (sc2, i2, p2a)) = deco_2

    self.inheritInterface(ni1, (sc1, i1))
    self.inheritInterface(ni2, (sc2, i2))

    f1 = self.getInterface(ni1).getFace()
    f2 = self.getInterface(ni2).getFace()

    p1o = f1.pts2d[p1a]
    p2o = f2.pts2d[p2a]

    for pts, mode in f1.decorations:
        transformDecorations(
            f2,
            [(px - p1o[0], py - p1o[1]) for (px, py) in pts],
            offset=p2o,
            rotate=angle,
            mode=mode
        )